@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003-2007  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2008  Hanna Knutsson (hanna_k@fmgirl.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -359,7 +359,7 @@ Argument *MathFunction::getArgumentDefinition(size_t index) {
 	return NULL;
 }
 void MathFunction::clearArgumentDefinitions() {
-	for(Sgi::hash_map<size_t, Argument*>::iterator it = argdefs.begin(); it != argdefs.end(); ++it) {
+	for(unordered_map<size_t, Argument*>::iterator it = argdefs.begin(); it != argdefs.end(); ++it) {
 		delete it->second;
 	}
 	argdefs.clear();
@@ -444,7 +444,7 @@ int MathFunction::parse(MathStructure &mstruct, const string &argv, const ParseO
 }
 bool MathFunction::testArguments(MathStructure &vargs) {
 	size_t last = 0;
-	for(Sgi::hash_map<size_t, Argument*>::iterator it = argdefs.begin(); it != argdefs.end(); ++it) {
+	for(unordered_map<size_t, Argument*>::iterator it = argdefs.begin(); it != argdefs.end(); ++it) {
 		if(it->first > last) {
 			last = it->first;
 		}
@@ -482,8 +482,8 @@ MathStructure MathFunction::calculate(MathStructure &vargs, const EvaluationOpti
 			}
 			return createFunctionMathStructureFromVArgs(vargs);
 		}
-		if(precision() > 0 && precision() < mstruct.precision()) mstruct.setPrecision(precision());
-		if(isApproximate()) mstruct.setApproximate();
+		if(precision() > 0 && (precision() < mstruct.precision() || mstruct.precision() < 1)) mstruct.setPrecision(precision(), true);
+		if(isApproximate()) mstruct.setApproximate(true, true);
 		return mstruct;
 	} else {
 		return createFunctionMathStructureFromVArgs(vargs);
@@ -864,12 +864,12 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		for(size_t i = 0; i < v_id.size(); i++) {
 			CALCULATOR->delId(v_id[i]);
 		}
-		if(precision() > 0) mstruct.setPrecision(precision());
-		if(isApproximate()) mstruct.setApproximate();
+		if(precision() > 0) mstruct.setPrecision(precision(), true);
+		if(isApproximate()) mstruct.setApproximate(true, true);
 	} else {
 		CALCULATOR->parse(&mstruct, sformula_calc, po);
-		if(precision() > 0) mstruct.setPrecision(precision());
-		if(isApproximate()) mstruct.setApproximate();
+		if(precision() > 0) mstruct.setPrecision(precision(), true);
+		if(isApproximate()) mstruct.setApproximate(true, true);
 	}
 	return 1;
 }
@@ -1925,6 +1925,11 @@ string AngleArgument::print() const {return _("angle");}
 string AngleArgument::subprintlong() const {return _("an angle or a number (using the default angle unit)");}
 void AngleArgument::parse(MathStructure *mstruct, const string &str, const ParseOptions &po) const {
 	CALCULATOR->parse(mstruct, str, po);
+	if(po.angle_unit != ANGLE_UNIT_NONE) {
+		if(mstruct->contains(CALCULATOR->getDegUnit(), false, true, true) > 0) return;
+		if(mstruct->contains(CALCULATOR->getGraUnit(), false, true, true) > 0) return;
+		if(mstruct->contains(CALCULATOR->getRadUnit(), false, true, true) > 0) return;
+	}
 	switch(po.angle_unit) {
 		case ANGLE_UNIT_DEGREES: {
 			mstruct->multiply(CALCULATOR->getDegUnit());
