@@ -1,7 +1,7 @@
 /*
-    Qalculate    
+    Qalculate (libary)
 
-    Copyright (C) 2003-2007  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,9 +54,11 @@ Unit::Unit(string cat_, string name_, string plural_, string singular_, string t
 		names[names.size() - 1].plural = true;
 	}
 	b_si = false;
+	b_use_with_prefixes = false;
 }
 Unit::Unit() {
 	b_si = false;
+	b_use_with_prefixes = false;
 }
 Unit::Unit(const Unit *unit) {
 	set(unit);
@@ -78,6 +80,7 @@ bool Unit::isSIUnit() const {
 void Unit::setAsSIUnit() {
 	if(!b_si) {
 		b_si = true;
+		b_use_with_prefixes = true;
 		ssystem == "SI";
 		setChanged(true);
 	}
@@ -87,6 +90,10 @@ void Unit::setSystem(string s_system) {
 		ssystem = s_system;
 		if(ssystem == "SI" || ssystem == "si" || ssystem == "Si") {
 			b_si = true;
+			b_use_with_prefixes = true;
+		} else if(ssystem == "CGS" || ssystem == "cgs" || ssystem == "Cgs") {
+			b_si = false;
+			b_use_with_prefixes = true;
 		} else {
 			b_si = false;
 		}
@@ -96,6 +103,12 @@ void Unit::setSystem(string s_system) {
 const string &Unit::system() const {
 	return ssystem;
 }
+bool Unit::useWithPrefixesByDefault() const {
+	return b_use_with_prefixes;
+}
+void Unit::setUseWithPrefixesByDefault(bool use_with_prefixes) {
+	b_use_with_prefixes = use_with_prefixes;
+}
 bool Unit::isCurrency() const {
 	return baseUnit() == CALCULATOR->u_euro;
 }
@@ -184,8 +197,8 @@ bool Unit::convert(Unit *u, MathStructure &mvalue, MathStructure &mexp) const {
 	} else if(u->baseUnit() == baseUnit()) {
 		u->convertToBaseUnit(mvalue, mexp);
 		convertFromBaseUnit(mvalue, mexp);
-		if(isCurrency()) {
-			CALCULATOR->checkExchangeRatesDate();
+		if(isCurrency() && u->isCurrency() && ((isBuiltin() && this != CALCULATOR->u_euro) || (u->isBuiltin() && u != CALCULATOR->u_euro))) {
+			CALCULATOR->setExchangeRatesUsed();
 		}
 		return true;
 	}
@@ -310,6 +323,7 @@ MathStructure AliasUnit::convertFromBaseUnit() const {
 int AliasUnit::baseExponent(int exp) const {
 	return o_unit->baseExponent(exp * i_exp);
 }
+
 MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
 	if(i_exp != 1) mexp /= i_exp;
 	ParseOptions po;
@@ -332,9 +346,29 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			CALCULATOR->parse(&mvalue, stmp, po);
 			CALCULATOR->delId(x_id);
 			CALCULATOR->delId(y_id);
+			if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+			if(isApproximate()) mvalue.setApproximate(true, true);
 		} else {
 			MathStructure *mstruct = new MathStructure();
 			CALCULATOR->parse(mstruct, svalue, po);
+			if(precision() > 0) {
+				if(mstruct->isNumber()) {
+					if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
+						mstruct->number().setPrecision(precision());
+						mstruct->numberUpdated();
+					}
+				} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
+					if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+						mstruct->getChild(1)->number().setPrecision(precision());
+						mstruct->getChild(1)->numberUpdated();
+						mstruct->childUpdated(1);
+					}
+				} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+					mstruct->setPrecision(precision(), true);
+				}
+			} else if(isApproximate() && !mstruct->isApproximate()) {
+				mstruct->setApproximate(true, true);
+			}
 			if(!mexp.isOne()) mstruct->raise(mexp);
 			mvalue.divide_nocopy(mstruct, true);
 		}
@@ -353,16 +387,34 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			gsub("\\y", stmp2, stmp);
 			CALCULATOR->parse(&mvalue, stmp, po);
 			CALCULATOR->delId(x_id);
-			CALCULATOR->delId(y_id);			
+			CALCULATOR->delId(y_id);
+			if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+			if(isApproximate()) mvalue.setApproximate(true, true);
 		} else {
 			MathStructure *mstruct = new MathStructure();
 			CALCULATOR->parse(mstruct, sinverse, po);
+			if(precision() > 0) {
+				if(mstruct->isNumber()) {
+					if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
+						mstruct->number().setPrecision(precision());
+						mstruct->numberUpdated();
+					}
+				} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
+					if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+						mstruct->getChild(1)->number().setPrecision(precision());
+						mstruct->getChild(1)->numberUpdated();
+						mstruct->childUpdated(1);
+					}
+				} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+					mstruct->setPrecision(precision(), true);
+				}
+			} else if(isApproximate() && !mstruct->isApproximate()) {
+				mstruct->setApproximate(true, true);
+			}
 			if(!mexp.isOne()) mstruct->raise(mexp);
 			mvalue.multiply_nocopy(mstruct, true);
 		}
-	}
-	if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision());
-	if(isApproximate()) mvalue.setApproximate();
+	}	
 	return mvalue;
 }
 MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
@@ -374,25 +426,50 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 		string stmp = svalue;
 		string stmp2 = LEFT_PARENTHESIS ID_WRAP_LEFT;
 		int x_id = CALCULATOR->addId(new MathStructure(mvalue), true);
+		int y_id = -1;
 		stmp2 += i2s(x_id);
 		stmp2 += ID_WRAP_RIGHT RIGHT_PARENTHESIS;
 		gsub("\\x", stmp2, stmp);
-		stmp2 = LEFT_PARENTHESIS ID_WRAP_LEFT;
-		int y_id = CALCULATOR->addId(new MathStructure(mexp), true);
-		stmp2 += i2s(y_id);
-		stmp2 += ID_WRAP_RIGHT RIGHT_PARENTHESIS;
-		gsub("\\y", stmp2, stmp);
+		if(svalue.find("\\y") != string::npos) {
+			stmp2 = LEFT_PARENTHESIS ID_WRAP_LEFT;
+			y_id = CALCULATOR->addId(new MathStructure(mexp), true);
+			stmp2 += i2s(y_id);
+			stmp2 += ID_WRAP_RIGHT RIGHT_PARENTHESIS;
+			gsub("\\y", stmp2, stmp);
+		}
 		CALCULATOR->parse(&mvalue, stmp, po);
 		CALCULATOR->delId(x_id);
-		CALCULATOR->delId(y_id);
+		if(y_id < 0) {
+			mvalue.raise(mexp);
+		} else {
+			CALCULATOR->delId(y_id);
+		}
+		if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+		if(isApproximate()) mvalue.setApproximate(true, true);
 	} else {
 		MathStructure *mstruct = new MathStructure();
 		CALCULATOR->parse(mstruct, svalue, po);
+		if(precision() > 0) {
+			if(mstruct->isNumber()) {
+				if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
+					mstruct->number().setPrecision(precision());
+					mstruct->numberUpdated();
+				}
+			} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
+				if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+					mstruct->getChild(1)->number().setPrecision(precision());
+					mstruct->getChild(1)->numberUpdated();
+					mstruct->childUpdated(1);
+				}
+			} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+				mstruct->setPrecision(precision(), true);
+			}
+		} else if(isApproximate() && !mstruct->isApproximate()) {
+			mstruct->setApproximate(true, true);
+		}		
 		if(!mexp.isOne()) mstruct->raise(mexp);
 		mvalue.multiply_nocopy(mstruct, true);
 	}
-	if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision());
-	if(isApproximate()) mvalue.setApproximate();
 	if(i_exp != 1) mexp.multiply(i_exp);
 	return mvalue;
 }
@@ -672,13 +749,15 @@ bool CompositeUnit::containsRelativeTo(Unit *u) const {
 	}	
 	return false;
 }
-MathStructure CompositeUnit::generateMathStructure(bool make_division) const {
+MathStructure CompositeUnit::generateMathStructure(bool make_division, bool set_null_prefixes) const {
 	MathStructure mstruct;
-	bool has_p = false;
-	for(size_t i = 0; i < units.size(); i++) {
-		if(units[i]->prefix()) {
-			has_p = true;
-			break;
+	bool has_p = set_null_prefixes;
+	if(!set_null_prefixes) {
+		for(size_t i = 0; i < units.size(); i++) {
+			if(units[i]->prefix()) {
+				has_p = true;
+				break;
+			}
 		}
 	}
 	MathStructure mden;
@@ -733,7 +812,8 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 	ParseOptions po;
 	po.variables_enabled = false;
 	po.functions_enabled = false;
-	po.unknowns_enabled = false;
+	po.unknowns_enabled = true;
+	po.convert_temperature_units = false;
 	MathStructure mstruct;
 	bool had_errors = false;
 	CALCULATOR->beginTemporaryStopMessages();
@@ -750,6 +830,16 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 				add(mstruct[i].unit(), 1, mstruct[i].prefix());
 			} else if(mstruct[i].isPower() && mstruct[i][0].isUnit() && mstruct[i][1].isInteger()) {
 				add(mstruct[i][0].unit(), mstruct[i][1].number().intValue(), mstruct[i][0].prefix());
+			} else if(mstruct[i].isMultiplication()) {
+				for(size_t i2 = 0; i2 < mstruct.size(); i2++) {
+					if(mstruct[i][i2].isUnit()) {
+						add(mstruct[i][i2].unit(), 1, mstruct[i][i2].prefix());
+					} else if(mstruct[i][i2].isPower() && mstruct[i][i2][0].isUnit() && mstruct[i][i2][1].isInteger()) {
+						add(mstruct[i][i2][0].unit(), mstruct[i][i2][1].number().intValue(), mstruct[i][i2][0].prefix());
+					} else {
+						had_errors = true;
+					}
+				}
 			} else {
 				had_errors = true;
 			}
