@@ -1,7 +1,7 @@
 /*
-    Qalculate    
+    Qalculate (library)
 
-    Copyright (C) 2003-2007  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,9 +17,11 @@
 
 #include <glib.h>
 #include <time.h>
+#include <utime.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
-#include <pwd.h>
+#include <fstream>
 
 
 bool eqstr::operator()(const char *s1, const char *s2) const {
@@ -60,7 +62,7 @@ void now(int &hour, int &min, int &sec) {
 }
 void today(int &year, int &month, int &day) {
 	GDate *gtime = g_date_new();
-	g_date_set_time(gtime, time(NULL));
+	g_date_set_time_t(gtime, time(NULL));
 	year = g_date_get_year(gtime);
 	month = g_date_get_month(gtime);
 	day = g_date_get_day(gtime);
@@ -162,10 +164,10 @@ int week(string str, bool start_sunday) {
 	GDate *gtime = g_date_new();
 	bool b = false;
 	if(str == _("today") || str == "today") {
-		g_date_set_time(gtime, time(NULL));
+		g_date_set_time_t(gtime, time(NULL));
 		b = true;
 	} else if(str == _("now") || str == "now") {
-		g_date_set_time(gtime, time(NULL));
+		g_date_set_time_t(gtime, time(NULL));
 		b = true;
 	} else {
 		b = s2date(str, gtime);
@@ -209,11 +211,11 @@ int weekday(string str) {
 	GDate *gtime = g_date_new();
 	bool b = false;
 	if(str == _("today") || str == "today") {
-		g_date_set_time(gtime, time(NULL));
+		g_date_set_time_t(gtime, time(NULL));
 		b = true;
 	} else if(str == _("now") || str == "now") {
-		g_date_set_time(gtime, time(NULL));
-		b = true;	
+		g_date_set_time_t(gtime, time(NULL));
+		b = true;
 	} else {
 		b = s2date(str, gtime);
 	}
@@ -229,10 +231,10 @@ int yearday(string str) {
 	GDate *gtime = g_date_new();
 	bool b = false;
 	if(str == _("today") || str == "today") {
-		g_date_set_time(gtime, time(NULL));
+		g_date_set_time_t(gtime, time(NULL));
 		b = true;
 	} else if(str == _("now") || str == "now") {
-		g_date_set_time(gtime, time(NULL));
+		g_date_set_time_t(gtime, time(NULL));
 		b = true;
 	} else {
 		b = s2date(str, gtime);
@@ -777,14 +779,61 @@ void parse_qalculate_version(string qalculate_version, int *qalculate_version_nu
 	}
 }
 
-string getLocalDir() {
-	string homedir = "";
-	struct passwd *pw = getpwuid(getuid());
-	if(pw) {
-		homedir = pw->pw_dir;
-		homedir += "/";
-	}
-	homedir += ".qalculate/";
+string getOldLocalDir() {
+	gchar *gstr = g_build_filename(g_get_home_dir(), ".qalculate", NULL);
+	string homedir = gstr;
+	g_free(gstr);
 	return homedir;
 }
+string getLocalDir() {
+	gchar *gstr = g_build_filename(g_get_user_config_dir(), "qalculate", NULL);
+	string confdir = gstr;
+	g_free(gstr);
+	return confdir;
+}
+string getLocalDataDir() {
+	gchar *gstr = g_build_filename(g_get_user_data_dir(), "qalculate", NULL);
+	string datadir = gstr;
+	g_free(gstr);
+	return datadir;
+}
+string getLocalTmpDir() {
+	gchar *gstr = g_build_filename(g_get_tmp_dir(), "qalculate", NULL);
+	string tmpdir = gstr;
+	g_free(gstr);
+	return tmpdir;
+}
+bool move_file(const char *from_file, const char *to_file) {
+
+	ifstream source(from_file);
+	if(source.fail()) {
+		source.close();
+		return false;
+	}
+
+	ofstream dest(to_file);	
+	if(dest.fail()) {
+		source.close();
+		dest.close();	
+		return false;
+	}
+
+	dest << source.rdbuf();
+
+	source.close();
+	dest.close();
+	
+	struct stat stats_from;
+	if(stat(from_file, &stats_from) == 0) {
+		struct utimbuf to_times;
+		to_times.actime = stats_from.st_atime;
+		to_times.modtime = stats_from.st_mtime;
+		utime(to_file, &to_times);
+	}
+	
+	remove(from_file);
+	
+	return true;
+	
+}
 
