@@ -1,7 +1,7 @@
 /*
-    Qalculate    
+    Qalculate
 
-    Copyright (C) 2004  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2004  Hanna Knutsson (hanna_k@fmgirl.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 #include <time.h>
 #include <pthread.h>
 #include <dirent.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <vector>
 #include <glib.h>
